import 'dart:async';

import 'package:flutter/services.dart';
typedef Future<dynamic> MessageHandler(String message);

class FlutterUmengPush {
  factory FlutterUmengPush() => _instance;
  FlutterUmengPush.private(MethodChannel channel)
      : _channel = channel;
  static final FlutterUmengPush _instance = new FlutterUmengPush.private(
      const MethodChannel('flutter_umpush'));

  final MethodChannel _channel;

  MessageHandler _onReceiveMessage;
  MessageHandler _onMessage;
  MessageHandler _onLaunch;
  MessageHandler _onResume;
  MessageHandler _onToken;

  /// Sets up [MessageHandler] for incoming messages.
  void configure({
    MessageHandler onReceiveMessage,
    MessageHandler onMessage,
    MessageHandler onLaunch,
    MessageHandler onResume,
    MessageHandler onToken,
  }) {
    _onReceiveMessage=onReceiveMessage;
    _onMessage = onMessage;
    _onLaunch = onLaunch;
    _onResume = onResume;
    _onToken = onToken;
    _channel.setMethodCallHandler(_handleMethod);
    _channel.invokeMethod('configure');
  }

  void test() {
    _channel.invokeMethod('test');
  }

  Future<Null> _handleMethod(MethodCall call) async {
    switch (call.method) {
      case "onToken":
        final String token = call.arguments;
        print('FlutterUmpush onToken: $token');
        _onToken(token);
        return null;
      case "onMessage":
        final String message = call.arguments;
        print('FlutterUmpush onMessage: $message');
        _onMessage(message);
        return null;
      case "onLaunch":
        final String message = call.arguments;
        print('FlutterUmpush onLaunch: $message');
        _onLaunch(call.arguments.cast<String>());
        return null;
      case "onResume":
        final String message = call.arguments;
        print('FlutterUmpush onResume: $message');
        _onResume(call.arguments.cast<String>());
        return null;
      case "onReceiveMessage":
        final String message = call.arguments;
        print('FlutterUmpush onReceiveMessage: $message');
        _onReceiveMessage(message);
        return null;
      default:
        throw new UnsupportedError("Unrecognized JSON message");
    }
  }
}
